"""Common configuration constants
"""

PROJECTNAME = 'Products.PFGMasterSelect'

ADD_PERMISSIONS = {
    # -*- extra stuff goes here -*-
    'FormMasterSelectStringField': 'Products.PFGMasterSelect: Add FormMasterSelectStringField',
    'FormMasterMultiSelectStringField': 'Products.PFGMasterSelect: Add FormMasterMultiSelectStringField',
}
