from zope.interface import Interface


class IFormMasterSelectStringField(Interface):
    """Description of the Example Type"""


class IFormMasterMultiSelectStringField(Interface):
    """Description of the Example Type"""
